@@ -1,3 +1,25 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001 Sun Microsystems Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <glib/gmessages.h>
 #include <glib/gslist.h>
 
@@ -11,12 +33,12 @@ typedef struct {
 static GSList *working_list = NULL; /* of Iteration */
 
 /*
- *   deletes an element from the list - in a re-enterant
+ *   deletes an element from the list - in a re-entrant
  * safe fashion; advances the element pointer to the next
  * element.
  */
 void
-spi_re_enterant_list_delete_link (GList * const *element_ptr)
+spi_re_entrant_list_delete_link (GList * const *element_ptr)
 {
   GSList    *l;
   GList     *next;
@@ -52,9 +74,9 @@ spi_re_enterant_list_delete_link (GList * const *element_ptr)
 }
 
 void
-spi_re_enterant_list_foreach (GList         **list,
-			      SpiReEnterantFn func,
-			      gpointer        user_data)
+spi_re_entrant_list_foreach (GList         **list,
+			     SpiReEntrantFn  func,
+			     gpointer        user_data)
 {
 	Iteration i;
 
